---
title: "Mastodon documentation revamp"
summary: "Reorganizing the documentation for the Mastodon Project, while also rewriting significant portions of it."
author: "Abdullah Tarawneh"
date: "2020-01-04"
tags: ["mastodon", "documentation", "information architecture", "rest api"]
category: "Work"
cover: "/images/mastodocs.jpg"
---

## Overview

The project had two phases -- first, I had to identify what was missing or could be improved from the old docs; second, I had to come up with a new structure and write all that documentation.

## Information architecture

### User guide

```
Old structure: "User Guide"
- Basics
- Decentralization
- Privacy
- Moderation

New structure: "Using Mastodon"
- Signing up for a new account
- Setting up your profile
- Posting toots
- Using the network features
- Dealing with unwanted content
- Promoting yourself and others
- Set your preferences
- More settings
- Using Mastodon externally
- Moving or leaving accounts
```

Many of the pages in the old user guide did not actually deal with teaching people how to use Mastodon, so the content within them was reworked into a more general landing page that explained the benefits of Mastodon. This new landing page explains the basics of microblogging and federation, before moving into the practical implications that these decisions have for user freedom, privacy, and safety. Select quotations were included from previously published promotional material on the Mastodon blog.

After this, an entirely new user guide was outlined from scratch, covering the user life cycle from start to end (or new beginning).

### Admin guide

```
Old structure: "Admin guide"
- Installation
- Configuration
- Post-installation steps
- Scaling up
- Optional features
- Upgrading to a new release
- Migrating servers
- Troubleshooting

New structure: "Running Mastodon"
- Preparing your machine
- Installing from source
- Configuring your environment
- Installing optional features
- Setting up your new instance
- Using the admin CLI
- Upgrading to a new release
- Backing up your server
- Migrating to a new machine
- Scaling up your server
- Moderation actions
- Troubleshooting errors
```

This section was mostly straightened out already, but could still use some improvements. A pre-installation page was split out from the old installation page. Backup instructions were moved out of "Optional features" and into a dedicated page. Pages were added for moderation and CLI usage.

### Development guide

```
Old structure: "Development guide"
- Overview
- ActivityPub compliance

New structure:
"Contributing to Mastodon"
- Technical overview
- Setting up a dev environment
- Code structure
- Routes
"Spec compliance"
- ActivityPub
- WebFinger
- Security
- Microformats
- OAuth
```

The old "overview" page contained a mixture of information about setting up a dev environment, libraries used, code structure, and useful commands to run for testing.

The first step in cleaning this up was to split this page into multiple discrete pages, each with its own purpose. As "development" is a vague term, two new sections were created to replace it: one for client development (merged with API), and one for server development. The old "overview" pages were nested under the server development section, as they dealt primarily with server development.

Next, "ActivityPub compliance" was moved into a dedicated section for spec compliance, and pages were created for documenting how various significant specifications were used and implemented within Mastodon.

### API

```
Old structure:
"API overview"
- Guidelines
- Libraries
- Authentication
- Permissions
- Entities
- Parameters
- Streaming API
- Web Push API
"REST API"
- Accounts
- Apps
- ...
- Timelines

New structure:
"Developing Mastodon Apps"
- Getting started with the API
- Playing with public data
- Obtaining client app access
- Logging in with an account
- Guidelines and best practices
- Libraries and implementations
"REST API"
- OAuth scopes
"API Methods"
- apps
  - oauth
- accounts
  - bookmarks
  - favourites
  - ...
- statuses
  - media
  - ...
- timelines
  - conversations
  - ...
  - streaming
- notifications
  - push
- search
- instance
  - trends
  - directory
  - custom_emoji
- admin
- proofs
- oembed
"API Entities"
- Account
- ...
- Token
```

This was the majority of the work. Mastodon's REST API documentation was really messy, and it showed. Finding something generally entailed flipping back and forth between multiple pages and searching within them, and each page was very long.

The first step was to promote methods and entities into their own sections. The "Entities" page was split into multiple pages, one per entity, and alphabetized. Tables were converted into headings, to allow for providing more information about each field.

Methods were grouped by namespace rather than by feature-set, and with one level of nesting depending on whether the methods within pertained to accounts, statuses, timelines, notifications, etc. The pages for the Streaming and Web Push APIs were moved out of the overview section and into the methods section.

Finally, a new section was created for a client development guide. The guide would cover the basics of REST API, how to make requests, how responses are structured, and authentication/authorization.

## Technical writing

### User guide

With the information architecture phase done, writing the user guide was a straightforward task. Explanations of each feature and setting were added to the appropriate pages, as well as screenshots demonstrating proper usage. Tip boxes are included throughout in order to highlight important points.

### Client development guide

Writing this section taught me a lot about the basics of REST APIs, understanding HTTP requests and responses, providing parameters, and how OAuth works -- all information that I included in the client development guide.

### Routes

While documenting the REST API, I had to consult the Ruby on Rails routing config file extensively, so I took it as an opportunity to write a page about how routes work and how to read the routes file.

### Specification compliance

While the basics of ActivityPub federation were already written, the format of the old document simply pointed toward the server-to-server spec, as well as HTTP Signatures and Linked Data Signatures (with no real explanation beyond that). Therefore, I rewrote this page into multiple separate pages, each detailing the spec in question. The "ActivityPub" page now covered status federation and profile federation, the properties and types used in each, HTML payload sanitization, namespacing, and extensions (with sample payloads). "WebFinger" explained what, why, and how to use WebFinger for actor URI resolution. "Security" explained HTTP and LD signatures. "Microformats" explained the use of Microformats 2.0 and how they may be parsed, and "OAuth" covered the endpoints and flows implemented within Mastodon for obtaining a token.

### REST API

Methods now include the HTTP verb, endpoint, description, return type, OAuth scope, version history, request parameters, and sample response code and payload.

Entities now include example payloads, as well as eacha ttribute and its description, type, and version history.

## Outcomes

> "Bless you for being here to work on the docs btw. It's a big relief."

### Less missing information

During the information architecture phase, a new skeleton was created as a proposed alternative structure. This process made the existing gaps in the old structure more obvious, and therefore those gaps could be filled during the technical writing phase.

### Easier to make additions in an appropriate place

Pages that were added after completion:

- "Rate limits" was added under "REST API", describing how to deal with rate limits on requests made to the REST API.
- "Bug bounties and responsible disclosure" was added under "Contributing to Mastodon", describing where serious issues should be reported if found.
- "Running your own server" was added under "Using Mastodon", describing the reasons why a user might want to run their own server and linking to the "Running Mastodon" section. 

The clear structure left in place by my information architecture work meant that it was almost immediately clear where to add these pages. 